package com.recordingblogs.squarewave;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.event.*;
import com.recordingblogs.orgenerator.*;

public class SquareWavePanel extends JPanel implements GeneratorPanelInterface, ActionListener, 
	FocusListener, ChangeListener
{
	/**
	 * The serial version UID, not used
	 */
	private static final long serialVersionUID = 160827874846847482L;
	
	private JTextField m_frequencyField;
	private JSlider m_frequencySlider;
	
	private SquareWave m_squareWave;

    public class SliderLabel extends JLabel
    {
		/**
		 * The serial version UID, not used
		 */
		private static final long serialVersionUID = -2209265095713750871L;

		public SliderLabel(String string)
        {
            super(string);
            setFont(GeneratorFont.SMALLFONT);
        }
    }

    private class FocusPolicy extends ContainerOrderFocusTraversalPolicy
    {
		/**
		 * The serial version UID, not used
		 */
		private static final long serialVersionUID = 5093326584239232604L;

		public Component getDefaultComponent(Container c)
        {
            return m_frequencyField;
        }

        public Component getLastComponent(Container c)
        {
            return m_frequencySlider;
        }

        public Component getFirstComponent(Container c)
        {
            return m_frequencyField;
        }

        public Component getComponentBefore(Container c, Component a)
        {
        	if (a == m_frequencyField)
        		return m_frequencySlider;
        	if (a == m_frequencySlider)
        		return m_frequencyField;
            return m_frequencyField;
        }

        public Component getComponentAfter(Container c, Component a)
        {
        	if (a == m_frequencyField)
        		return m_frequencySlider;
        	if (a == m_frequencySlider)
        		return m_frequencyField;
            return m_frequencyField;
        }
    }

    public SquareWavePanel(SquareWave squarewave)
    {
        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(layout);

        JLabel frequencyLabel = new JLabel("Frequency (Hz):");
        frequencyLabel.setFont(GeneratorFont.LARGEFONT);
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.weightx = 0;
        constraints.weighty = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(frequencyLabel, constraints);
        add(frequencyLabel);
        
        m_frequencyField = new JTextField(6);
        m_frequencyField.setToolTipText("Set the frequency");
        m_frequencyField.addActionListener(this);
        m_frequencyField.addFocusListener(this);
        constraints.gridx = 2;
        constraints.insets.left = 5;
        layout.setConstraints(m_frequencyField, constraints);
        add(m_frequencyField);

        Hashtable<Integer, SliderLabel> numbersFrequency = new Hashtable<Integer, SliderLabel>();
        numbersFrequency.put(Integer.valueOf(0), new SliderLabel("0"));
        numbersFrequency.put(Integer.valueOf(200), new SliderLabel("80"));
        numbersFrequency.put(Integer.valueOf(400), new SliderLabel("330"));
        numbersFrequency.put(Integer.valueOf(600), new SliderLabel("1,350"));
        numbersFrequency.put(Integer.valueOf(800), new SliderLabel("5,400"));
        numbersFrequency.put(Integer.valueOf(1000), new SliderLabel("22,000"));

        m_frequencySlider = new JSlider(JSlider.HORIZONTAL, 
        		(int)(1000F * Math.log(SquareWave.MINFREQUENCY / SquareWave.MINFREQUENCY) / Math.log(SquareWave.MAXFREQUENCY / SquareWave.MINFREQUENCY)), 
        		(int)(1000F * Math.log(SquareWave.MAXFREQUENCY / SquareWave.MINFREQUENCY) / Math.log(SquareWave.MAXFREQUENCY / SquareWave.MINFREQUENCY)), 
        		0);
        m_frequencySlider.setMajorTickSpacing(200);
        m_frequencySlider.setMinorTickSpacing(50);
        m_frequencySlider.setPaintTicks(true);
        m_frequencySlider.setPaintLabels(true);
        m_frequencySlider.setFont(GeneratorFont.SMALLFONT);
        m_frequencySlider.setLabelTable(numbersFrequency);
        m_frequencySlider.setToolTipText("Set the frequency");
        m_frequencySlider.addChangeListener(this);
        m_frequencySlider.addFocusListener(this);
        constraints.gridx = 3;
        constraints.weightx = 1.0D;
        layout.setConstraints(m_frequencySlider, constraints);
        add(m_frequencySlider);

        m_squareWave = squarewave;
        
        setFocusTraversalPolicy(new FocusPolicy());
        updateData();
    }

    public void stateChanged(ChangeEvent e)
    {
        if(e.getSource() == m_frequencySlider)
        {
            m_squareWave.setFrequency((float) (SquareWave.MINFREQUENCY * Math.pow(SquareWave.MAXFREQUENCY / SquareWave.MINFREQUENCY, m_frequencySlider.getValue() / 1000F)));
            updateData();
        }
    }

    public void actionPerformed(ActionEvent e)
    {
        if(e.getSource() == m_frequencyField)
        {
            try
            {
            	float value = Float.parseFloat(m_frequencyField.getText());
            	if (value < SquareWave.MINFREQUENCY || value > SquareWave.MAXFREQUENCY)
            	{
                    m_frequencySlider.grabFocus();
                    return;
            	}
	            m_squareWave.setFrequency(value);
	            updateData();
            }
            catch (NumberFormatException ex)
            {
            	m_frequencySlider.grabFocus();
            }
        }
    }

    public void focusGained(FocusEvent e)
    {
    }

    public void focusLost(FocusEvent e)
    {
        if(e.getSource() == m_frequencyField)
        {
        	try
        	{
            	float value = Float.parseFloat(m_frequencyField.getText());
            	if (value < SquareWave.MINFREQUENCY || value > SquareWave.MAXFREQUENCY)
            	{
                    JOptionPane.showMessageDialog(this, "Please enter a number between " + (int) SquareWave.MINFREQUENCY + " and "
                    		+ (int) SquareWave.MAXFREQUENCY, "Error", JOptionPane.ERROR_MESSAGE);
                    m_frequencyField.grabFocus();
                    updateData();
                    return;
            	}
	            m_squareWave.setFrequency(value);
	            updateData();
        	}
        	catch (NumberFormatException ex)
        	{
                JOptionPane.showMessageDialog(this, "Please enter a valid number", 
                		"Error", JOptionPane.ERROR_MESSAGE);
	            updateData();
        		m_frequencyField.grabFocus();
        	}
        }
        else if(e.getSource() == m_frequencySlider)
        {
            m_squareWave.setFrequency((float) (SquareWave.MINFREQUENCY * Math.pow(SquareWave.MAXFREQUENCY / SquareWave.MINFREQUENCY, m_frequencySlider.getValue() / 1000F)));
        }
    }

    public void updateData()
    {
        m_frequencySlider.setValue((int)(1000F * Math.log(m_squareWave.getFrequency() / SquareWave.MINFREQUENCY) / Math.log(SquareWave.MAXFREQUENCY / SquareWave.MINFREQUENCY)));
        m_frequencyField.setText(Integer.toString((int) m_squareWave.getFrequency()));
    }
}
